@@ -13,6 +13,6 @@ cargl(long double complex z)
        #ifdef _LDBL_EQ_DBL
          return carg (z);
        #else
-         return atan2l (cimagl (z), creall (z));
+         return atan2l (imag (z), real (z));
        #endif
 }
