@@ -363,21 +363,22 @@ fhandler_console::read (void *pv, size_t& buflen)
 	  goto err;		/* seems to be failure */
 	}
 
+      const WCHAR &unicode_char = input_rec.Event.KeyEvent.uChar.UnicodeChar;
+      const DWORD &ctrl_key_state = input_rec.Event.KeyEvent.dwControlKeyState;
+
       /* check the event that occurred */
       switch (input_rec.EventType)
 	{
 	case KEY_EVENT:
-#define virtual_key_code (input_rec.Event.KeyEvent.wVirtualKeyCode)
-#define control_key_state (input_rec.Event.KeyEvent.dwControlKeyState)
 
 	  con.nModifiers = 0;
 
 #ifdef DEBUGGING
 	  /* allow manual switching to/from raw mode via ctrl-alt-scrolllock */
-	  if (input_rec.Event.KeyEvent.bKeyDown &&
-	      virtual_key_code == VK_SCROLL &&
-	      ((control_key_state & (LEFT_ALT_PRESSED | LEFT_CTRL_PRESSED)) == (LEFT_ALT_PRESSED | LEFT_CTRL_PRESSED))
-	    )
+	  if (input_rec.Event.KeyEvent.bKeyDown
+	      && input_rec.Event.KeyEvent.wVirtualKeyCode == VK_SCROLL
+	      && (ctrl_key_state & (LEFT_ALT_PRESSED | LEFT_CTRL_PRESSED))
+		  == (LEFT_ALT_PRESSED | LEFT_CTRL_PRESSED))
 	    {
 	      set_raw_win32_keyboard_mode (!con.raw_win32_keyboard_mode);
 	      continue;
@@ -398,31 +399,23 @@ fhandler_console::read (void *pv, size_t& buflen)
 	      break;
 	    }
 
-#define ich (input_rec.Event.KeyEvent.uChar.AsciiChar)
-#define wch (input_rec.Event.KeyEvent.uChar.UnicodeChar)
-
-	  /* Ignore key up events, except for left alt events with non-zero character
-	   */
+	  /* Ignore key up events, except for Alt+Numpad events. */
 	  if (!input_rec.Event.KeyEvent.bKeyDown &&
-	      /*
-		Event for left alt, with a non-zero character, comes from
-		"alt + numerics" key sequence.
-		e.g. <left-alt> 0233 => &eacute;
-	      */
-	      !(wch != 0
-		// ?? experimentally determined on an XP system
-		&& virtual_key_code == VK_MENU
-		// left alt -- see http://www.microsoft.com/hwdev/tech/input/Scancode.asp
-		&& input_rec.Event.KeyEvent.wVirtualScanCode == 0x38))
+	      !is_alt_numpad_event (&input_rec))
+	    continue;
+	  /* Ignore Alt+Numpad keys.  They are eventually handled below after
+	     releasing the Alt key. */
+	  if (input_rec.Event.KeyEvent.bKeyDown
+	      && is_alt_numpad_key (&input_rec))
 	    continue;
 
-	  if (control_key_state & SHIFT_PRESSED)
+	  if (ctrl_key_state & SHIFT_PRESSED)
 	    con.nModifiers |= 1;
-	  if (control_key_state & RIGHT_ALT_PRESSED)
+	  if (ctrl_key_state & RIGHT_ALT_PRESSED)
 	    con.nModifiers |= 2;
-	  if (control_key_state & CTRL_PRESSED)
+	  if (ctrl_key_state & CTRL_PRESSED)
 	    con.nModifiers |= 4;
-	  if (control_key_state & LEFT_ALT_PRESSED)
+	  if (ctrl_key_state & LEFT_ALT_PRESSED)
 	    con.nModifiers |= 8;
 
 	  /* Allow Backspace to emit ^? and escape sequences. */
@@ -430,7 +423,7 @@ fhandler_console::read (void *pv, size_t& buflen)
 	    {
 	      char c = con.backspace_keycode;
 	      nread = 0;
-	      if (control_key_state & ALT_PRESSED)
+	      if (ctrl_key_state & ALT_PRESSED)
 		{
 		  if (con.metabit)
 		    c |= 0x80;
@@ -443,10 +436,10 @@ fhandler_console::read (void *pv, size_t& buflen)
 	    }
 	  /* Allow Ctrl-Space to emit ^@ */
 	  else if (input_rec.Event.KeyEvent.wVirtualKeyCode == VK_SPACE
-		   && (control_key_state & CTRL_PRESSED)
-		   && !(control_key_state & ALT_PRESSED))
+		   && (ctrl_key_state & CTRL_PRESSED)
+		   && !(ctrl_key_state & ALT_PRESSED))
 	    toadd = "";
-	  else if (wch == 0
+	  else if (unicode_char == 0
 	      /* arrow/function keys */
 	      || (input_rec.Event.KeyEvent.dwControlKeyState & ENHANCED_KEY))
 	    {
@@ -460,17 +453,17 @@ fhandler_console::read (void *pv, size_t& buflen)
 	    }
 	  else
 	    {
-	      nread = con.con_to_str (tmp + 1, 59, wch);
+	      nread = con.con_to_str (tmp + 1, 59, unicode_char);
 	      /* Determine if the keystroke is modified by META.  The tricky
 		 part is to distinguish whether the right Alt key should be
 		 recognized as Alt, or as AltGr. */
 	      bool meta =
 		     /* Alt but not AltGr (= left ctrl + right alt)? */
-		     (control_key_state & ALT_PRESSED) != 0
-		     && ((control_key_state & CTRL_PRESSED) == 0
+		     (ctrl_key_state & ALT_PRESSED) != 0
+		     && ((ctrl_key_state & CTRL_PRESSED) == 0
 			    /* but also allow Alt-AltGr: */
-			 || (control_key_state & ALT_PRESSED) == ALT_PRESSED
-			 || (wch <= 0x1f || wch == 0x7f));
+			 || (ctrl_key_state & ALT_PRESSED) == ALT_PRESSED
+			 || (unicode_char <= 0x1f || unicode_char == 0x7f));
 	      if (!meta)
 		{
 		  /* Determine if the character is in the current multibyte
@@ -498,8 +491,6 @@ fhandler_console::read (void *pv, size_t& buflen)
 		  con.nModifiers &= ~4;
 		}
 	    }
-#undef ich
-#undef wch
 	  break;
 
 	case MOUSE_EVENT:
@@ -799,7 +790,7 @@ fhandler_console::scroll_buffer_screen (int x1, int y1, int x2, int y2, int xn,
   if (y1 >= 0)
     y1 += con.b.srWindow.Top;
   if (y2 >= 0)
-    y1 += con.b.srWindow.Top;
+    y2 += con.b.srWindow.Top;
   if (yn >= 0)
     yn += con.b.srWindow.Top;
   con.scroll_buffer (get_output_handle (), x1, y1, x2, y2, xn, yn);
@@ -2005,6 +1996,8 @@ fhandler_console::write_normal (const unsigned char *src,
 
   /* The alternate charset is always 437, just as in the Linux console. */
   f_mbtowc = con.get_console_cp () ? __cp_mbtowc (437) : __MBTOWC;
+  if (f_mbtowc == __ascii_mbtowc)
+    f_mbtowc = __utf8_mbtowc;
 
   /* First check if we have cached lead bytes of a former try to write
      a truncated multibyte sequence.  If so, process it. */
