@@ -46,7 +46,7 @@ struct _fpstate
   __uint32_t padding[24];
 };
 
-struct __attribute__ ((__aligned__ (16))) __mcontext
+struct __attribute__ ((aligned (16))) __mcontext
 {
   __uint64_t p1home;
   __uint64_t p2home;
@@ -187,10 +187,7 @@ struct _sigcommune
   void *_si_process_handle;
   __extension__ union
   {
-    struct {
-      int      _si_fd;
-      uint32_t _si_flags;
-    };
+    int _si_fd;
     int64_t _si_pipe_unique_id;
     char *_si_str;
   };
@@ -268,10 +265,11 @@ enum
   SI_MESGQ,				/* sent by real time mesq state change
 					   (currently unimplemented) */
   SI_TIMER,				/* sent by timer expiration */
-  SI_QUEUE,				/* sent by sigqueue */
+  SI_QUEUE,				/* sent by sigqueue (currently
+					   unimplemented) */
   SI_KERNEL,				/* sent by system */
 
-  ILL_ILLOPC = 7,			/* illegal opcode */
+  ILL_ILLOPC,				/* illegal opcode */
   ILL_ILLOPN,				/* illegal operand */
   ILL_ILLADR,				/* illegal addressing mode */
   ILL_ILLTRP,				/* illegal trap*/
@@ -280,7 +278,7 @@ enum
   ILL_COPROC,				/* coprocessor error */
   ILL_BADSTK,				/* internal stack error */
 
-  FPE_INTDIV = 15,			/* integer divide by zero */
+  FPE_INTDIV,				/* integer divide by zero */
   FPE_INTOVF,				/* integer overflow */
   FPE_FLTDIV,				/* floating point divide by zero */
   FPE_FLTOVF,				/* floating point overflow */
@@ -289,14 +287,14 @@ enum
   FPE_FLTINV,				/* floating point invalid operation */
   FPE_FLTSUB,				/* subscript out of range */
 
-  SEGV_MAPERR = 23,			/* address not mapped to object */
+  SEGV_MAPERR,				/* address not mapped to object */
   SEGV_ACCERR,				/* invalid permissions for mapped object */
 
-  BUS_ADRALN = 25,			/* invalid address alignment.  */
+  BUS_ADRALN,				/* invalid address alignment.  */
   BUS_ADRERR,				/* non-existant physical address.  */
   BUS_OBJERR,				/* object specific hardware error.  */
 
-  CLD_EXITED = 28,			/* child has exited */
+  CLD_EXITED,				/* child has exited */
   CLD_KILLED,				/* child was killed */
   CLD_DUMPED,				/* child terminated abnormally */
   CLD_TRAPPED,				/* traced child has trapped */
@@ -304,40 +302,6 @@ enum
   CLD_CONTINUED				/* stopped child has continued */
 };
 
-#define SI_USER SI_USER
-#define SI_ASYNCIO SI_ASYNCIO
-#define SI_MESGQ SI_MESGQ
-#define SI_TIMER SI_TIMER
-#define SI_QUEUE SI_QUEUE
-#define SI_KERNEL SI_KERNEL
-#define ILL_ILLOPC ILL_ILLOPC
-#define ILL_ILLOPN ILL_ILLOPN
-#define ILL_ILLADR ILL_ILLADR
-#define ILL_ILLTRP ILL_ILLTRP
-#define ILL_PRVOPC ILL_PRVOPC
-#define ILL_PRVREG ILL_PRVREG
-#define ILL_COPROC ILL_COPROC
-#define ILL_BADSTK ILL_BADSTK
-#define FPE_INTDIV FPE_INTDIV
-#define FPE_INTOVF FPE_INTOVF
-#define FPE_FLTDIV FPE_FLTDIV
-#define FPE_FLTOVF FPE_FLTOVF
-#define FPE_FLTUND FPE_FLTUND
-#define FPE_FLTRES FPE_FLTRES
-#define FPE_FLTINV FPE_FLTINV
-#define FPE_FLTSUB FPE_FLTSUB
-#define SEGV_MAPERR SEGV_MAPERR
-#define SEGV_ACCERR SEGV_ACCERR
-#define BUS_ADRALN BUS_ADRALN
-#define BUS_ADRERR BUS_ADRERR
-#define BUS_OBJERR BUS_OBJERR
-#define CLD_EXITED CLD_EXITED
-#define CLD_KILLED CLD_KILLED
-#define CLD_DUMPED CLD_DUMPED
-#define CLD_TRAPPED CLD_TRAPPED
-#define CLD_STOPPED CLD_STOPPED
-#define CLD_CONTINUED CLD_CONTINUED
-
 enum
 {
   SIGEV_SIGNAL = 0,			/* a queued signal, with an application
@@ -438,18 +402,14 @@ struct sigaction
 #define	SIGUSR2 31	/* user defined signal 2 */
 
 #if __WORDSIZE == 64
-#define _NSIG	65      /* signal 0 implied */
+#define NSIG	65      /* signal 0 implied */
 #else
-#define _NSIG	33      /* signal 0 implied */
-#endif
-
-#if __MISC_VISIBLE
-#define NSIG	_NSIG
+#define NSIG	33      /* signal 0 implied */
 #endif
 
 /* Real-Time signals per SUSv3.  RT_SIGMAX is defined as 8 in limits.h */
 #define SIGRTMIN 32
-#define SIGRTMAX (_NSIG - 1)
+#define SIGRTMAX (NSIG - 1)
 
 #define SIG_HOLD ((_sig_func_ptr)2)	/* Signal in signal mask */
 
