@@ -68,7 +68,7 @@ __weak_alias(strptime,_strptime)
 #define ALT_O			0x02
 #define	LEGAL_ALT(x)		{ if (alt_format & ~(x)) return NULL; }
 
-static const int _DAYS_BEFORE_MONTH[12] =
+static _CONST int _DAYS_BEFORE_MONTH[12] =
 {0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334};
 
 #define SET_MDAY 1
@@ -413,15 +413,13 @@ literal:
 		case 'F':	/* The date as "%Y-%m-%d". */
 			{
 			  LEGAL_ALT(0);
+			  ymd |= SET_YMD;
 			  char *tmp = __strptime ((const char *) bp, "%Y-%m-%d",
 						  tm, era_info, alt_digits,
 						  locale);
-			  /* width max chars converted, default 10, < 6 => 6 */
-			  if (tmp && (char *) bp +
-				(!width ? 10 : width < 6 ? 6 : width) < tmp)
+			  if (tmp && (uint) (tmp - (char *) bp) > width)
 			    return NULL;
 			  bp = (const unsigned char *) tmp;
-			  ymd |= SET_YMD;
 			  continue;
 			}
 
@@ -575,26 +573,6 @@ literal:
 			bp = conv_num(bp, &tm->tm_sec, 0, 61, ALT_DIGITS);
 			continue;
 
-		case 's' :	/* The seconds since Unix epoch - GNU extension */
-		    {
-			long long sec;
-			time_t t;
-			char *end;
-			save_errno save;
-
-			LEGAL_ALT(0);
-			sec = strtoll_l ((char *)bp, &end, 10, locale);
-			t = sec;
-			if (end == (char *)bp
-			    || errno != 0
-			    || t != sec
-			    || localtime_r (&t, tm) != tm)
-			    return NULL;
-			bp = (const unsigned char *)end;
-			ymd |= SET_YDAY | SET_WDAY | SET_YMD;
-			break;
-		    }
-
 		case 'U':	/* The week of year, beginning on sunday. */
 		case 'W':	/* The week of year, beginning on monday. */
 			/*
@@ -703,8 +681,8 @@ literal:
 				const unsigned char *ep;
 
 				ep = find_string(bp, &i,
-						 (const char * const *)tzname,
-						  NULL, 2, locale);
+					       	 (const char * const *)tzname,
+					       	  NULL, 2, locale);
 				if (ep != NULL) {
 					tm->tm_isdst = i;
 #ifdef TM_GMTOFF
@@ -750,7 +728,7 @@ literal:
 	    /* Check if year falls into the era.  If not, it's an
 	       invalid combination of era and offset. */
 	    if (era->start.tm_year > tm->tm_year
-		|| era->end.tm_year < tm->tm_year)
+	    	|| era->end.tm_year < tm->tm_year)
 	      return NULL;
 	    tm->tm_year -= TM_YEAR_BASE;
 	  }
