@@ -19,7 +19,15 @@ details. */
 #include "ntdll.h"
 #include "tls_pbuf.h"
 #include "cpuid.h"
-#include "clock.h"
+
+static long
+get_open_max (int in)
+{
+  long max = getdtablesize ();
+  if (max < OPEN_MAX)
+    max = OPEN_MAX;
+  return max;
+}
 
 static long
 get_page_size (int in)
@@ -332,11 +340,9 @@ get_cpu_cache_intel_cpuid4 (int in)
 	case _SC_LEVEL1_ICACHE_ASSOC:
 	  if (cur_level == 1 && cache_type == 2)
 	    return assoc;
-	  break;
 	case _SC_LEVEL1_ICACHE_LINESIZE:
 	  if (cur_level == 1 && cache_type == 2)
 	    return linesize;
-	  break;
 	case _SC_LEVEL1_DCACHE_SIZE:
 	  if (cur_level == 1 && cache_type == 1)
 	    ret += assoc * part * linesize * sets;
@@ -344,11 +350,9 @@ get_cpu_cache_intel_cpuid4 (int in)
 	case _SC_LEVEL1_DCACHE_ASSOC:
 	  if (cur_level == 1 && cache_type == 1)
 	    return assoc;
-	  break;
 	case _SC_LEVEL1_DCACHE_LINESIZE:
 	  if (cur_level == 1 && cache_type == 1)
 	    return linesize;
-	  break;
 	case _SC_LEVEL2_CACHE_SIZE:
 	  if (cur_level == 2)
 	    ret += assoc * part * linesize * sets;
@@ -356,11 +360,9 @@ get_cpu_cache_intel_cpuid4 (int in)
 	case _SC_LEVEL2_CACHE_ASSOC:
 	  if (cur_level == 2)
 	    return assoc;
-	  break;
 	case _SC_LEVEL2_CACHE_LINESIZE:
 	  if (cur_level == 2)
 	    return linesize;
-	  break;
 	case _SC_LEVEL3_CACHE_SIZE:
 	  if (cur_level == 3)
 	    ret += assoc * part * linesize * sets;
@@ -368,11 +370,9 @@ get_cpu_cache_intel_cpuid4 (int in)
 	case _SC_LEVEL3_CACHE_ASSOC:
 	  if (cur_level == 3)
 	    return assoc;
-	  break;
 	case _SC_LEVEL3_CACHE_LINESIZE:
 	  if (cur_level == 3)
 	    return linesize;
-	  break;
 	}
     }
   return ret;
@@ -485,8 +485,7 @@ get_cpu_cache (int in)
   vendor_id[3] = 0;
   if (!strcmp ((char*) vendor_id, "GenuineIntel"))
     return get_cpu_cache_intel (in, maxf & 0xffff);
-  else if (!strcmp ((char*)vendor_id, "AuthenticAMD")
-           || !strcmp((char*)vendor_id, "HygonGenuine"))
+  else if (!strcmp ((char*)vendor_id, "AuthenticAMD"))
     {
       uint32_t maxe = 0, unused;
       cpuid (&maxe, &unused, &unused, &unused, 0x80000000);
@@ -511,7 +510,7 @@ static struct
   {cons, {c:CHILD_MAX}},		/*   1, _SC_CHILD_MAX */
   {cons, {c:CLOCKS_PER_SEC}},		/*   2, _SC_CLK_TCK */
   {cons, {c:NGROUPS_MAX}},		/*   3, _SC_NGROUPS_MAX */
-  {cons, {c:OPEN_MAX}},		/*   4, _SC_OPEN_MAX */
+  {func, {f:get_open_max}},		/*   4, _SC_OPEN_MAX */
   {cons, {c:_POSIX_JOB_CONTROL}},	/*   5, _SC_JOB_CONTROL */
   {cons, {c:_POSIX_SAVED_IDS}},		/*   6, _SC_SAVED_IDS */
   {cons, {c:_POSIX_VERSION}},		/*   7, _SC_VERSION */
@@ -528,7 +527,7 @@ static struct
   {cons, {c:SIGQUEUE_MAX}},		/*  18, _SC_SIGQUEUE_MAX */
   {cons, {c:TIMER_MAX}},		/*  19, _SC_TIMER_MAX */
   {nsup, {c:0}},			/*  20, _SC_TZNAME_MAX */
-  {cons, {c:_POSIX_ASYNCHRONOUS_IO}},	/*  21, _SC_ASYNCHRONOUS_IO */
+  {cons, {c:-1L}},			/*  21, _SC_ASYNCHRONOUS_IO */
   {cons, {c:_POSIX_FSYNC}},		/*  22, _SC_FSYNC */
   {cons, {c:_POSIX_MAPPED_FILES}},	/*  23, _SC_MAPPED_FILES */
   {cons, {c:-1L}},			/*  24, _SC_MEMLOCK */
@@ -541,9 +540,9 @@ static struct
   {cons, {c:_POSIX_SHARED_MEMORY_OBJECTS}},	/*  31, _SC_SHARED_MEMORY_OBJECTS */
   {cons, {c:_POSIX_SYNCHRONIZED_IO}},	/*  32, _SC_SYNCHRONIZED_IO */
   {cons, {c:_POSIX_TIMERS}},		/*  33, _SC_TIMERS */
-  {cons, {c:AIO_LISTIO_MAX}},		/*  34, _SC_AIO_LISTIO_MAX */
-  {cons, {c:AIO_MAX}},			/*  35, _SC_AIO_MAX */
-  {cons, {c:AIO_PRIO_DELTA_MAX}},	/*  36, _SC_AIO_PRIO_DELTA_MAX */
+  {nsup, {c:0}},			/*  34, _SC_AIO_LISTIO_MAX */
+  {nsup, {c:0}},			/*  35, _SC_AIO_MAX */
+  {nsup, {c:0}},			/*  36, _SC_AIO_PRIO_DELTA_MAX */
   {nsup, {c:0}},			/*  37, _SC_DELAYTIMER_MAX */
   {cons, {c:PTHREAD_KEYS_MAX}},		/*  38, _SC_THREAD_KEYS_MAX */
   {cons, {c:PTHREAD_STACK_MIN}},	/*  39, _SC_THREAD_STACK_MIN */
@@ -589,7 +588,7 @@ static struct
   {cons, {c:SYMLOOP_MAX}},		/*  79, _SC_SYMLOOP_MAX */
   {cons, {c:_POSIX_THREAD_CPUTIME}},	/*  80, _SC_THREAD_CPUTIME */
   {cons, {c:-1L}},			/*  81, _SC_THREAD_SPORADIC_SERVER */
-  {cons, {c:_POSIX_TIMEOUTS}},		/*  82, _SC_TIMEOUTS */
+  {cons, {c:-1L}},			/*  82, _SC_TIMEOUTS */
   {cons, {c:-1L}},			/*  83, _SC_TRACE */
   {cons, {c:-1L}},			/*  84, _SC_TRACE_EVENT_FILTER */
   {nsup, {c:0}},			/*  85, _SC_TRACE_EVENT_NAME_MAX */
@@ -720,14 +719,10 @@ static struct
   {ls ("")},				/* _CS_POSIX_V7_THREADS_CFLAGS */
   {ls ("")},				/* _CS_POSIX_V7_THREADS_LDFLAGS */
   {ls ("POSIXLY_CORRECT=1")},		/* _CS_V7_ENV */
-  {ls ("")},				/* _CS_LFS_CFLAGS */
-  {ls ("")},				/* _CS_LFS_LDFLAGS */
-  {ls ("")},				/* _CS_LFS_LIBS */
-  {ls ("")},				/* _CS_LFS_LINTFLAGS */
 };
 
 #define CS_MIN _CS_PATH
-#define CS_MAX _CS_LFS_LINTFLAGS
+#define CS_MAX _CS_V7_ENV
 
 extern "C" size_t
 confstr (int in, char *buf, size_t len)
@@ -794,7 +789,7 @@ sysinfo (struct sysinfo *info)
 				     sizeof_stodi, NULL);
   if (NT_SUCCESS (status))
     uptime = (stodi->CurrentTime.QuadPart - stodi->BootTime.QuadPart)
-	     / NS100PERSEC;
+	     / 10000000ULL;
   else
     debug_printf ("NtQuerySystemInformation(SystemTimeOfDayInformation), "
 		  "status %y", status);
