@@ -17,9 +17,6 @@ details. */
 /* resource.cc */
 extern size_t get_rlimit_stack (void);
 
-/* thread.cc */
-char *mythreadname (void);
-
 #include <pthread.h>
 #include <limits.h>
 #include "security.h"
@@ -111,20 +108,7 @@ typedef enum
 } verifyable_object_state;
 
 template <class list_node> inline void
-List_insert (fast_mutex &mx, list_node *&head, list_node *node)
-{
-  if (!node)
-    return;
-  mx.lock ();
-  do
-    node->next = head;
-  while (InterlockedCompareExchangePointer ((PVOID volatile *) &head,
-					    node, node->next) != node->next);
-  mx.unlock ();
-}
-
-template <class list_node> inline void
-List_insert_nolock (list_node *&head, list_node *node)
+List_insert (list_node *&head, list_node *node)
 {
   if (!node)
     return;
@@ -176,7 +160,7 @@ template <class list_node> class List
 
   void insert (list_node *node)
   {
-    List_insert (mx, head, node);
+    List_insert (head, node);
   }
 
   void remove (list_node *node)
@@ -210,7 +194,6 @@ protected:
 class pthread_key: public verifyable_object
 {
   DWORD tls_index;
-  static bool iterate_dtors_once_more;
 public:
   static bool is_good_object (pthread_key_t const *);
 
@@ -232,24 +215,7 @@ public:
 
   static void run_all_destructors ()
   {
-    /* POSIX requires at least four iterations of running destructors:
-
-       If, after all the destructors have been called for all non-NULL
-       values with associated destructors, there are still some non-NULL
-       values with associated destructors, then the process is repeated.
-       If, after at least {PTHREAD_DESTRUCTOR_ITERATIONS} iterations of
-       destructor calls for outstanding non-NULL values, there are still
-       some non-NULL values with associated destructors, implementations
-       may stop calling destructors, or they may continue calling
-       destructors until no non-NULL values with associated destructors
-       exist, even though this might result in an infinite loop. */
-    for (int i = 0; i < PTHREAD_DESTRUCTOR_ITERATIONS; ++i)
-      {
-	iterate_dtors_once_more = false;
-	keys.for_each (&pthread_key::run_destructor);
-	if (!iterate_dtors_once_more)
-	  break;
-      }
+    keys.for_each (&pthread_key::run_destructor);
   }
 
   /* List support calls */
@@ -301,7 +267,7 @@ public:
   static bool is_initializer_or_object (pthread_mutex_t const *);
   static bool is_initializer_or_bad_object (pthread_mutex_t const *);
 
-  int lock (PLARGE_INTEGER timeout = NULL);
+  int lock ();
   int trylock ();
   int unlock ();
   int destroy ();
@@ -406,7 +372,7 @@ public:
 
   /* API calls */
   static int cancel (pthread_t);
-  static int join (pthread_t * thread, void **return_val, PLARGE_INTEGER);
+  static int join (pthread_t * thread, void **return_val);
   static int detach (pthread_t * thread);
   static int create (pthread_t * thread, const pthread_attr_t * attr,
 			      void *(*start_routine) (void *), void *arg);
@@ -621,10 +587,10 @@ public:
   } *readers;
   fast_mutex readers_mx;
 
-  int rdlock (PLARGE_INTEGER timeout = NULL);
+  int rdlock ();
   int tryrdlock ();
 
-  int wrlock (PLARGE_INTEGER timeout = NULL);
+  int wrlock ();
   int trywrlock ();
 
   int unlock ();
@@ -692,8 +658,7 @@ public:
   static int post (sem_t *sem);
   static int getvalue (sem_t *sem, int *sval);
   static int trywait (sem_t *sem);
-  static int clockwait (sem_t *sem, clockid_t clock_id,
-			const struct timespec *abstime);
+  static int timedwait (sem_t *sem, const struct timespec *abstime);
 
   static int getinternal (sem_t *sem, int *sfd, unsigned long long *shash,
 			  LUID *sluid, unsigned int *sval);
@@ -728,10 +693,11 @@ public:
   }
 
 private:
+  int _wait ();
   void _post ();
   int _getvalue (int *sval);
   int _trywait ();
-  int _wait (PLARGE_INTEGER timeout = NULL);
+  int _timedwait (const struct timespec *abstime);
 
   void _fixup_before_fork ();
   void _fixup_after_fork ();
