@@ -186,6 +186,7 @@ static const struct conf_variable conf_table[] =
   { "POSIX2_RE_DUP_MAX",		CONSTANT,	_POSIX2_RE_DUP_MAX	},
 
   /* Variables from fpathconf() */
+  { "CASE_INSENSITIVE",			PATHCONF,	_PC_CASE_INSENSITIVE	},
   { "FILESIZEBITS",			PATHCONF,	_PC_FILESIZEBITS	},
   { "LINK_MAX",				PATHCONF,	_PC_LINK_MAX		},
   { "MAX_CANON",			PATHCONF,	_PC_MAX_CANON		},
@@ -195,10 +196,12 @@ static const struct conf_variable conf_table[] =
   { "PIPE_BUF",				PATHCONF,	_PC_PIPE_BUF		},
   { "POSIX2_SYMLINKS",			PATHCONF,	_PC_2_SYMLINKS		},
   { "POSIX_ALLOC_SIZE_MIN",		PATHCONF,	_PC_ALLOC_SIZE_MIN	},
+  { "POSIX_PERMISSIONS",		PATHCONF,	_PC_POSIX_PERMISSIONS	},
   { "POSIX_REC_INCR_XFER_SIZE",		PATHCONF,	_PC_REC_INCR_XFER_SIZE	},
   { "POSIX_REC_MAX_XFER_SIZE",		PATHCONF,	_PC_REC_MAX_XFER_SIZE	},
   { "POSIX_REC_MIN_XFER_SIZE",		PATHCONF,	_PC_REC_MIN_XFER_SIZE	},
   { "POSIX_REC_XFER_ALIGN",		PATHCONF,	_PC_REC_XFER_ALIGN	},
+  { "POSIX_SECURITY",			PATHCONF,	_PC_POSIX_SECURITY	},
   { "SYMLINK_MAX",			PATHCONF,	_PC_SYMLINK_MAX		},
   { "_POSIX_CHOWN_RESTRICTED",		PATHCONF,	_PC_CHOWN_RESTRICTED	},
   { "_POSIX_NO_TRUNC",			PATHCONF,	_PC_NO_TRUNC		},
